''  jm_font.bas - jm_font
''	Copyright (C) 2005-2006 Jeff Marshall (coder@execulink.com)
''
''	This program is free software; you can redistribute it and/or modify
''	it under the terms of the GNU General Public License as published by
''	the Free Software Foundation; either version 2 of the License, or
''	(at your option) any later version.
''
''	This program is distributed in the hope that it will be useful,
''	but WITHOUT ANY WARRANTY; without even the implied warranty of
''	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''	GNU General Public License for more details.
''
''	You should have received a copy of the GNU General Public License
''	along with this program; if not, write to the Free Software
''	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.

Option Explicit

#include "inc1/jm_font_rt.bi"
#include "jm_font.bi"
#include "../jm_string/jm_string.bi"

Dim Shared FONT_curfnt As JM_FONT_PTR

Function FONT_Get alias "FONT_Get" () As JM_FONT_PTR
	Return FONT_curfnt
End Function

Function FONT_Set alias "FONT_Set" (byval fnt As JM_FONT_PTR) As JM_FONT_PTR
	Dim lstfnt As JM_FONT_PTR
	lstfnt = FONT_curfnt
	FONT_curfnt = fnt
	Return lstfnt
End Function

Sub FONT_Delete alias "FONT_Delete" (byval fnt As JM_FONT_PTR)
	if (fnt) then
		if (fnt->delete) then
			fnt->delete(fnt)
		end if
		deallocate(fnt)
	end if
end sub

' =================
' Text Metrics
' =================

Function FONT_GetCharAdvanceX alias "FONT_GetCharAdvanceX" (byval ch as integer) as integer
	Dim fnt As JM_FONT_PTR = FONT_curfnt
	If (fnt) Then
		function = fnt->GetCharAdvanceX(fnt, ch)
	End If
End Function

Function FONT_GetCharAdvanceY alias "FONT_GetCharAdvanceY" (byval ch as integer) as integer
	Dim fnt As JM_FONT_PTR = FONT_curfnt
	If (fnt) Then
		function = fnt->GetCharAdvanceY(fnt, ch)
	End If
End Function

Function FONT_GetTextWidthN alias "FONT_GetTextWidthN" (ByVal s As ubyte Ptr, ByVal nBytes As Integer) As Integer
	Dim fnt As JM_FONT_PTR = FONT_curfnt, i as integer = 0, w as integer = 0
	If (fnt) Then
		while i < nBytes
			w += fnt->GetCharAdvanceX(fnt, cast(integer, s[i]))
			i += 1
		wend
		function = w
	End If
End Function

Function FONT_GetTextWidth alias "FONT_GetTextWidth" (byval s as zstring ptr) As Integer
	function = FONT_GetTextWidthN(s, zstr_Len(s))
End Function

Function FONT_GetTextHeightN alias "FONT_GetTextHeightN" (ByVal s As ubyte Ptr, ByVal nBytes As Integer) As Integer
	Dim fnt As JM_FONT_PTR = FONT_curfnt
	If (fnt) Then
		function = fnt->h
	End If
End Function

Function FONT_GetTextHeight alias "FONT_GetTextHeight" (byval s as zstring ptr) As Integer
	function = FONT_GetTextHeightN(s, zstr_Len(s))
End Function

Function FONT_GetIndexFromX alias "FONT_GetIndexFromX" (Byval s as uByte Ptr, ByVal nBytes as integer, ByVal StartIndex as integer, ByVal xPos as integer) as integer
	Dim fnt As JM_FONT_PTR = FONT_curfnt
	dim xx as integer = xPos
	dim i as integer = StartIndex
	if xPos > 0 then
		while i < nBytes
			xx -= fnt->GetCharAdvanceX(fnt, s[i])
			if xx <= 0 then
				exit while
			end if
			i += 1
		wend
	elseif xPos < 0 then
		while i > 0 and xx < 0
			xx += fnt->GetCharAdvanceX(fnt, s[i])
			if xx >= 0 then 
				exit while
			end if
			i -= 1
		wend
	end if
	function = i
end function

Function FONT_GetMaxChars alias "FONT_GetMaxChars" (Byval s as uByte Ptr, ByVal nBytes as integer, byval w as integer ) as integer
	Dim fnt As JM_FONT_PTR = FONT_curfnt
	dim i as integer = 0
	dim xx as integer = 0
	while i < nBytes
		xx += fnt->GetCharAdvanceX(fnt, s[i])
		if xx >= w then
			exit while
		end if
		i += 1
	wend
	function = i
end function

' =================
' STRING DRAWING
' =================

Function FONT_DrawChar alias "FONT_DrawChar"  (ByVal x as Integer, ByVal y as Integer, ByVal ch as Integer, ByVal foreColor as Integer, ByVal backColor as Integer, ByVal Trans as integer) as integer
	Dim fnt As JM_FONT_PTR = FONT_curfnt
	if (fnt) then
		function = fnt->DrawChar(fnt,x,y,ch,forecolor,backcolor,trans)
	end if	
End Function

Function FONT_DrawStringN alias "FONT_DrawStringN" (ByVal x as Integer, ByVal y as Integer, ByVal txt as zstring Ptr, ByVal nBytes As Integer, ByVal foreColor as Integer, ByVal backColor as Integer, ByVal Trans as integer) as integer
	Dim xx as Integer, i As Integer, n As Integer, c as ubyte ptr = txt, w as integer = 0
	Dim fnt As JM_FONT_PTR = FONT_curfnt
	if (fnt) then
		if nBytes > 0 then
			n = nBytes
		else
			n = zstr_Len(txt)
		end if
		For i = 0 to n - 1
			w += fnt->DrawChar(fnt,x+w, y, c[i], foreColor, backColor, trans)
		Next i
		function = w
	end if
End function

Function FONT_DrawString alias "FONT_DrawString" (ByVal x as Integer, ByVal y as Integer, ByVal txt as zstring Ptr, ByVal foreColor as Integer, ByVal backColor as Integer, ByVal Trans as integer) as integer
	'' FIXME: needed?
	'Dim x_ as Integer = x
	'Dim y_ as Integer = y
	'Dim txt_ as zstring Ptr = txt
	'Dim n_ as integer = zstr_Len(txt)
	'Dim foreColor_ as Integer = forecolor
	'Dim backColor_ as Integer = backcolor
	'Dim Trans_ as integer = trans
	'function = FONT_DrawStringN(x_,y_,txt_,n_,forecolor_,backcolor_,trans_)
	function = FONT_DrawStringN(x,y,txt,zstr_Len(txt),forecolor,backcolor,trans)
End function
