''  jm_boxes.bas - jm_gui
''	Copyright (C) 2005-2006 Jeff Marshall (coder@execulink.com)
''
''	This program is free software; you can redistribute it and/or modify
''	it under the terms of the GNU General Public License as published by
''	the Free Software Foundation; either version 2 of the License, or
''	(at your option) any later version.
''
''	This program is distributed in the hope that it will be useful,
''	but WITHOUT ANY WARRANTY; without even the implied warranty of
''	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''	GNU General Public License for more details.
''
''	You should have received a copy of the GNU General Public License
''	along with this program; if not, write to the Free Software
''	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.

Option Explicit

#include "inc1/jm_gui_rt.bi"
#include "jm_gui_base.bi"
#include "jm_boxes.bi"
#include "../jm_font/jm_font.bi"

'' -------------------------------------------------------------------
'' BOX CREATION
'' -------------------------------------------------------------------

Function UI_CreateRoot alias "UI_CreateRoot" (byval id As integer, _
	ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer ) As UI_BOX_PTR
	return UI_CreateBox(NULL, UI_BOX_ROOT, id, 0, 0, 0, 0, BoxProc, @UI_DefRootProc_cb, 0, flags Or UI_BOX_FLAG_ROOT)
End Function

Function UI_CreateFrame alias "UI_CreateFrame" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
	byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
	ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer ) As UI_BOX_PTR
	return UI_CreateBox(ownerbox, UI_BOX_FRAME, id, x, y, w, h, BoxProc, @UI_DefFrameProc_cb, 0, flags)
End Function

Function UI_CreateCaption alias "UI_CreateCaption" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
	byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
	ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer, byval strCaption as zstring ptr) As UI_BOX_PTR
	Dim tmp As UI_BOX_PTR
	tmp = UI_CreateBox(ownerbox, UI_BOX_CAPTION, id, x, y, w, h, BoxProc, @UI_DefCaptionProc_cb, 0, flags)
	If (tmp) Then
		UI_SetCaption(tmp, strCaption)
	End If
	return tmp
End Function

Function UI_CreateButton alias "UI_CreateButton" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
	byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
	ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer, byval strCaption As zstring ptr) As UI_BOX_PTR
	Dim tmp As UI_BOX_PTR
	tmp = UI_CreateBox(ownerbox, UI_BOX_BUTTON, id, x, y, w, h, BoxProc, @UI_DefButtonProc_cb, 0, flags)
	If (tmp) Then
		UI_SetCaption(tmp, strCaption)
	End If
End Function

Function UI_CreateCheck alias "UI_CreateCheck" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
	byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
	ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer, byval strCaption As zstring ptr) As UI_BOX_PTR
	Dim tmp As UI_BOX_PTR
	tmp = UI_CreateBox(ownerbox, UI_BOX_CHECK, id, x, y, w, h, BoxProc, @UI_DefCheckProc_cb, sizeof(UI_CHECK_T), flags)
	If (tmp) Then
		UI_SetCaption(tmp, strCaption)
	End If
End Function

'' -------------------------------------------------------------------
'' HELPERS
'' -------------------------------------------------------------------

Sub UI_SetCaptionById alias "UI_SetCaptionById" (ByVal owner As UI_BOX_PTR, ByVal id As Integer, txt As zString Ptr)
	Dim cap As UI_BOX_PTR
	cap = UI_GetBoxFromID(owner, id)
	If (cap) Then
		UI_SetCaption(cap, txt)
	End If
End Sub

'' -------------------------------------------------------------------
'' DEFAULT BOXPROCS
'' -------------------------------------------------------------------

'' FIXME: Check OWNER_DRAW Flags

Function UI_DefRootProc_cb alias "UI_DefRootProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
	return 1
End Function

Function UI_DefFrameProc_cb alias "UI_DefFrameProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
	Dim r As UI_RECT_T
	If (UI_CallBoxProc(box, msgid, msgdata)) Then
		Select Case msgid
		Case UI_MSG_PAINT
			UI_GetScreenCoords box, @r
			UI_DrawFrame @r, UI_FS_BTN_UP
		Case UI_MSG_MOUSE_DOWN
			if (box->flags And UI_BOX_FLAG_DRAG) <> 0 Then
				UI_BeginDrag(box, msgdata->mx, msgdata->my)
			End If
		Case UI_MSG_MOUSE_UP
			UI_EndDrag(box, msgdata->mx, msgdata->my)
		Case UI_MSG_MOUSE_MOVE
			UI_Drag(box, msgdata->mx, msgdata->my)
		case else
		  return 1 '' Msg not processed
		End Select
	else
	  return 1 '' Msg not processed
	End If
End Function

Function UI_DefCaptionProc_cb alias "UI_DefCaptionProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
	Dim r As UI_RECT_T
	If (UI_CallBoxProc(box, msgid, msgdata)) Then
		Select Case msgid
		Case UI_MSG_PAINT
			UI_GetScreenCoords box, @r
			UI_DrawStdCaption @r, box->flags, UI_GetCaption(box)
		CASE UI_MSG_MOUSE_DOWN
			If UI_GetCapture() = NULL Then
				UI_SetCapture box
			End If
		CASE UI_MSG_MOUSE_UP
			UI_ReleaseCapture
		Case Else
			return 1
		End Select
	else
	  return 1 '' Msg not processed
	End If
End Function

Function UI_DefButtonProc_cb alias "UI_DefButtonProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
	Dim r As UI_RECT_T

	If (UI_CallBoxProc(box, msgid, msgdata)) Then
		Select Case msgid
		Case UI_MSG_PAINT
			UI_GetScreenCoords box, @r
			UI_DrawStdButton @r, box->flags, UI_GetCaption(box)
		CASE UI_MSG_MOUSE_DOWN
			If UI_GetCapture() = NULL Then
				UI_SetCapture box
				box->flags or= UI_BOX_FLAG_DOWN
			End If
		CASE UI_MSG_MOUSE_UP
			box->flags and= (not UI_BOX_FLAG_DOWN)
			UI_ReleaseCapture
		Case UI_MSG_MOUSE_MOVE
			If UI_GetCapture() = box Then
				UI_GetBoxCoords box, @r
				If UI_IsPointInRect(@r, msgdata->mx, msgdata->my) Then
					box->flags or= UI_BOX_FLAG_DOWN
				Else
					box->flags and= (not UI_BOX_FLAG_DOWN)
				End If
			End If
		Case Else
			return 1
		End Select
	else
	  return 1 '' Msg not processed
	End If
End Function

Function UI_DefCheckProc_cb alias "UI_DefCheckProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
	Dim r As UI_RECT_T
	Dim d As UI_CHECK_T Ptr = UI_GetBoxExtraBytesPtr(box)

	If (UI_CallBoxProc(box, msgid, msgdata)) Then
		Select Case msgid
		Case UI_MSG_PAINT
			UI_GetScreenCoords box, @r
			UI_DrawStdCheck @r, box->flags, d, UI_GetCaption(box)
		CASE UI_MSG_MOUSE_DOWN
			If UI_GetCapture() = NULL Then
				UI_SetCapture box
				box->flags or= UI_BOX_FLAG_DOWN
			End If
		CASE UI_MSG_MOUSE_UP
			box->flags and= (not UI_BOX_FLAG_DOWN)
			If UI_GetCapture() = box Then
				UI_GetBoxCoords box, @r
				If UI_IsPointInRect(@r, msgdata->mx, msgdata->my) Then
					d->checked xor= 1
				End If
			End If
			UI_ReleaseCapture
		Case UI_MSG_MOUSE_MOVE
			If UI_GetCapture() = box Then
				UI_GetBoxCoords box, @r
				If UI_IsPointInRect(@r, msgdata->mx, msgdata->my) Then
					box->flags or= UI_BOX_FLAG_DOWN
				Else
					box->flags and= (not UI_BOX_FLAG_DOWN)
				End If
			End If
		Case Else
			return 1
		End Select
	else
	  return 1 '' Msg not processed
	End If

End Function

