''  jm_boxes_std_input.bas - jm_gui
''	Copyright (C) 2005-2006 Jeff Marshall (coder@execulink.com)
''
''	This program is free software; you can redistribute it and/or modify
''	it under the terms of the GNU General Public License as published by
''	the Free Software Foundation; either version 2 of the License, or
''	(at your option) any later version.
''
''	This program is distributed in the hope that it will be useful,
''	but WITHOUT ANY WARRANTY; without even the implied warranty of
''	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''	GNU General Public License for more details.
''
''	You should have received a copy of the GNU General Public License
''	along with this program; if not, write to the Free Software
''	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.

Option Explicit

#include "inc1/jm_gui_rt.bi"
#include "jm_gui_base.bi"
#include "jm_boxes.bi"
#include "../jm_font/jm_font.bi"
#include "jm_boxes_std_input.bi"
#include "jm_boxes_std_edit.bi"
#include "../jm_string/jm_string.bi"

'' -------------------------------------------------------------------
'' BOX CREATION
'' -------------------------------------------------------------------

Function UI_CreateInput alias "UI_CreateInput" (ByVal ownerbox As UI_BOX_PTR, byval id As integer, _
	byval x As Integer, byval y As Integer, byval w As Integer, byval h As Integer, _
	ByVal BoxProc As BOXPROC_FUNC_T, ByVal flags As Integer, byval strCaption As zstring ptr) As UI_BOX_PTR
	Dim tmp As UI_BOX_PTR
	tmp = UI_CreateBox(ownerbox, UI_BOX_INPUT, id, x, y, w, h, BoxProc, @UI_DefInputProc_cb, sizeof(UI_INPUT_T), flags)
	If (tmp) Then
		Dim d As UI_INPUT_T Ptr = UI_GetBoxExtraBytesPtr(tmp)
		dim b as LINE_BUFFER_T ptr
		b = d->buffer
		LINE_BUFFER_SetText(b,strCaption,zstr_Len(strCaption))
		''LINE_BUFFER_SetText(d->buffer,strCaption, zstr_Len(strCaption))  '' Why problem on linux
		LINE_BUFFER_ClearChanged(d->buffer)
	End If
End Function

'' -------------------------------------------------------------------
'' DEFAULT BOXPROCS
'' -------------------------------------------------------------------

Function UI_DefInputProc_cb alias "UI_DefInputProc_cb"  (ByVal box As UI_BOX_PTR, ByVal msgid As Integer, ByVal msgdata As UI_MSG_DATA_T Ptr) As Integer
	Dim r As UI_RECT_T
	Dim d As UI_INPUT_T Ptr = UI_GetBoxExtraBytesPtr(box)
	''dim cx as integer, cy as integer
	dim n as integer, xchars as integer

	If (UI_CallBoxProc(box, msgid, msgdata)) Then

		Select Case msgid
		Case UI_MSG_CREATE
			d->flags = 0
			d->buffer = LINE_BUFFER_Create(1024)
			d->leftindex = 0
		
		Case UI_MSG_DESTROY
			LINE_BUFFER_Destroy d->buffer
			d->buffer = 0
		
		Case UI_MSG_PAINT
			UI_GetScreenCoords box, @r

			UI_DrawFrame @r, UI_FS_EDIT

			UI_InflateRect @r, -1, -1

			'' Figure out how to display the text, adjust the d->leftindex
			'' if needed if d->buffer->mindex is outside the current text box

			'' check if d->buffer->mindex is inside the box
			if d->buffer->mindex < d->leftindex then
				d->leftindex = FONT_GetIndexFromX(d->buffer->mdata, d->buffer->mlength, d->buffer->mindex, - r.w \ 2)
			else
				if FONT_GetIndexFromX(d->buffer->mdata, d->buffer->mlength, d->leftindex, r.w) < d->buffer->mindex then
					d->leftindex = FONT_GetIndexFromX(d->buffer->mdata, d->buffer->mlength, d->buffer->mindex, - r.w \ 2)
				end if
			end if

			xchars = FONT_GetMaxChars(d->buffer->mdata + d->leftindex, d->buffer->mlength - d->leftindex, r.w)

			n = xchars
			if (d->buffer->mlength - d->leftindex) < xchars then
				n = d->buffer->mlength - d->leftindex
			end if

			if d->buffer->mlength > xchars then
				UI_DrawTextN @r, d->buffer->mdata + d->leftindex, n, UI_GetSysColor(UI_SYSCLR_TEXT), UI_GetSysColor(UI_SYSCLR_FRAME_FACE), UI_TEXT_ALIGN_LEFT
			else
				UI_DrawTextN @r, d->buffer->mdata + d->leftindex, d->buffer->mlength, UI_GetSysColor(UI_SYSCLR_TEXT), UI_GetSysColor(UI_SYSCLR_FRAME_FACE), UI_TEXT_ALIGN_LEFT ''UI_TEXT_ALIGN_CENTER
			end if

			if UI_GetFocus() = box then
				dim xx = r.x + FONT_GetTextWidthN(d->buffer->mdata + d->leftindex, d->buffer->mindex - d->leftindex)
				dim yy = r.y
				UI_DrawCaret xx,yy, _
					FONT_GetTextWidthN(d->buffer->mdata + d->buffer->mindex, 1), _
					FONT_GetTextHeightN(d->buffer->mdata + d->buffer->mindex, 1), _
					UI_GetSysColor(UI_SYSCLR_TEXT)
			end if

		case UI_MSG_MOUSE_DOWN
			if UI_GetCapture = NULL then
				UI_SetCapture box
				LINE_BUFFER_SetIndex(d->buffer, FONT_GetIndexFromX(d->buffer->mdata, d->buffer->mlength, d->leftindex, msgdata->mx))
			end if
		case UI_MSG_MOUSE_UP
			if UI_GetCapture() = box then
				LINE_BUFFER_SetIndex(d->buffer, FONT_GetIndexFromX(d->buffer->mdata, d->buffer->mlength, d->leftindex, msgdata->mx))
			end if
			UI_ReleaseCapture
		case UI_MSG_MOUSE_MOVE
			if UI_GetCapture() = box then
				LINE_BUFFER_SetIndex(d->buffer, FONT_GetIndexFromX(d->buffer->mdata, d->buffer->mlength, d->leftindex, msgdata->mx))
			end if

		Case UI_MSG_KEYPRESS
			dim buf as LINE_BUFFER_T Ptr = d->buffer
			dim ch as integer = msgdata->ky
			Select case ch
			case KEY_LEFT
				if buf->mindex > 0 then buf->mindex -= 1
			case KEY_RIGHT
				if buf->mindex < buf->mlength then buf->mindex += 1
			case KEY_HOME
				if buf->mindex > 0 then	buf->mindex = 0
			case KEY_END
				if buf->mindex < buf->mlength then buf->mindex = buf->mlength
			case KEY_BACK
				LINE_BUFFER_BackChar buf
			''case KEY_INSERT
			case KEY_DELETE
				LINE_BUFFER_DeleteChar buf
			case 32 to 126
				LINE_BUFFER_InsertChar buf, ch
			case else
				return 1
			end select
		Case Else
			return 1
		End Select
	else
	  return 1 '' Msg not processed
	End If
End Function


