''  jm_boxes_yagl.bas - jm_gui
''	Copyright (C) 2005-2006 Jeff Marshall (coder@execulink.com)
''
''	This program is free software; you can redistribute it and/or modify
''	it under the terms of the GNU General Public License as published by
''	the Free Software Foundation; either version 2 of the License, or
''	(at your option) any later version.
''
''	This program is distributed in the hope that it will be useful,
''	but WITHOUT ANY WARRANTY; without even the implied warranty of
''	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''	GNU General Public License for more details.
''
''	You should have received a copy of the GNU General Public License
''	along with this program; if not, write to the Free Software
''	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.

Option Explicit

#include "inc1/jm_gui_rt.bi"
#include "jm_gui_base.bi"
#include "jm_boxes.bi"
#include "yaglwrapper.bi"

declare function UI_MakeRoundBorderImage alias "UI_MakeRoundBorderImage" (byval clr as integer,byval invert as integer, byval sz as integer) As any ptr
declare function UI_MakeSquareBorderImage alias "UI_MakeSquareBorderImage" (byval clr as integer,byval invert as integer, byval sz as integer) As any ptr

const bs as integer = 16.0							' border size
const sz as integer = bs * 3.0         ' texture size
const th as integer = 4

Dim shared As YaglGfxSurface brdrup, brdrdn, squareup, squaredn

'' Stretch blit ranged using YaglGfxDevice_blitscaledranged for actual blit
private sub StretchBlitRanged alias "StretchBlitRanged" (byval x1 as integer,byval y1 as integer,byval x2 as integer,byval y2 as integer, _
  byval src_min_x as integer,byval src_min_y as integer,byval src_max_x as integer,byval src_max_y as integer, _
	byval surf As YaglGfxSurface, byval blit_mode as YAGL_BLIT_MODE, byval blending as Single)

	dim scalex as single = csng(x2 - x1 + 1.0) / csng(abs(src_max_x - src_min_x) + 1)
	dim scaley as single = csng(y2 - y1 + 1.0) / csng(abs(src_max_y - src_min_y) + 1)

	YaglGfxDevice_blitscaledranged x1,y1,scalex,scaley,src_min_x,src_min_y,src_max_x,src_max_y,surf,blit_mode,blending
end sub

private function MakeRoundBorderTex alias "MakeRoundBorderTex" (byval clr as integer,byval invert as integer) As YaglGfxSurface
	dim a as ubyte ptr, ret as YaglGfxSurface = 0
	a = UI_MakeRoundBorderImage(clr, invert, sz)
	if (a) then
		ret = YaglGfxDevice_createSurface( )	
		YaglGfxSurface_loadFromMemory(ret, Cast(byte ptr, a) + 4, sz, sz, BITMAP_FORMAT_ARGB32)
		Deallocate a
	end if
	function = ret
end function		

private function MakeSquareBorderTex alias "MakeSquareBorderTex" (byval clr as integer,byval invert as integer) As YaglGfxSurface
	dim a as ubyte ptr, ret as YaglGfxSurface = 0
	a = UI_MakeSquareBorderImage(clr, invert, sz)
	if (a) then
		ret = YaglGfxDevice_createSurface( )	
		YaglGfxSurface_loadFromMemory(ret, Cast(byte ptr, a) + 4, sz, sz, BITMAP_FORMAT_ARGB32)
		Deallocate a
	end if
	function = ret
end function		

'' draw border @ (x,y)-step(w,h) using surface `s' with a thickness of
private sub DrawBorder alias "DrawBorder" (byval x as integer, byval y as integer, byval w as integer, byval h as integer, _
	byval s as YaglGfxSurface, ByVal thickness as integer, byval blending as single = 1.0, _
	byval fill as integer = 1, byval outside as integer = 0, byval Tinting as integer = &hffffff)

	dim as integer s1 = 0
	dim as integer s2 = bs - 1 
	dim as integer s3 = bs
	dim as integer s4 = sz - 1 - bs - 1
	dim as integer s5 = sz - 1 - bs
	dim as integer s6 = sz - 1

	dim as integer x1 = x 
	dim as integer x2 = x - 1 + thickness
	dim as integer x3 = x + thickness
	dim as integer x4 = x + w - 1 - thickness
	dim as integer x5 = x + w - thickness
	dim as integer x6 = x + w - 1
	
	dim as integer y1 = y 
	dim as integer y2 = y - 1 + thickness
	dim as integer y3 = y + thickness
	dim as integer y4 = y + h - 1 - thickness
	dim as integer y5 = y + h - thickness
	dim as integer y6 = y + h - 1

	Dim prevTinting as integer 

	if (outside) then
		x1 -= thickness : x2 -= thickness : x3 -= thickness
		x4 += thickness : x5 += thickness : x6 += thickness
		y1 -= thickness : y2 -= thickness : y3 -= thickness
		y4 += thickness : y5 += thickness : y6 += thickness
	end if

	prevTinting = YaglGfxDevice_getTintingColor()
	YaglGfxDevice_setTintingColor(tinting)

	'' Top
	StretchBlitRanged  x1,y1,x2,y2,s1,s1,s2,s2, s, BLIT_ALPHAMASKED, blending
	StretchBlitRanged  x3,y1,x4,y2,s3,s1,s4,s2, s, BLIT_ALPHAMASKED, blending
	StretchBlitRanged  x5,y1,x6,y2,s5,s1,s6,s2, s, BLIT_ALPHAMASKED, blending

	'' Mid
	StretchBlitRanged  x1,y3,x2,y4,s1,s3,s2,s4, s, BLIT_ALPHAMASKED, blending
	if (fill) then
	StretchBlitRanged  x3,y3,x4,y4,s3,s3,s4,s4, s, BLIT_ALPHAMASKED, blending
	end if
	StretchBlitRanged  x5,y3,x6,y4,s5,s3,s6,s4, s, BLIT_ALPHAMASKED, blending

	'' Bottom
	StretchBlitRanged  x1,y5,x2,y6,s1,s5,s2,s6, s, BLIT_ALPHAMASKED, blending
	StretchBlitRanged  x3,y5,x4,y6,s3,s5,s4,s6, s, BLIT_ALPHAMASKED, blending
	StretchBlitRanged  x5,y5,x6,y6,s5,s5,s6,s6, s, BLIT_ALPHAMASKED, blending

	YaglGfxDevice_setTintingColor(prevTinting)

end sub

Sub UI_DrawInit alias "UI_DrawInit" ()
	'if squareup then
	'	YaglGfxDevice_DestroySurface squareup
	'end if
	'if squaredn then
	'	YaglGfxDevice_DestroySurface squareup
	'end if
	'if brdrup then
	'	YaglGfxDevice_DestroySurface squareup
	'end if
	'if brdrdn then
	'	YaglGfxDevice_DestroySurface squareup
	'end if
	squareup = MakeSquareBorderTex(Rgb(255,255,255),0)
	squaredn = MakeSquareBorderTex(Rgb(255,255,255),1)
	brdrup = MakeRoundBorderTex(Rgb(255,255,255),0)
	brdrdn = MakeRoundBorderTex(Rgb(255,255,255),1)
End Sub

'' -------------------------------------------------------------------
'' DRAWING PRIMITIVES
'' -------------------------------------------------------------------

''private Sub UI_DrawLine(ByVal x1 As Integer, ByVal y1 As Integer, ByVal x2 As Integer, ByVal y2 As Integer, ByVal clr As Integer)
''	YaglGfxDevice_line x1,y1,x2,y2,clr,1.0
''End Sub
''private Sub UI_DrawHLine(ByVal x As Integer, ByVal y As Integer, Byval w As Integer, ByVal clr As Integer)
''	YaglGfxDevice_line x,y,x+w-1,y,clr,1.0
''End Sub
''private Sub UI_DrawVLine(ByVal x As Integer, ByVal y As Integer, Byval h As Integer, ByVal clr As Integer)
''	YaglGfxDevice_line x,y,x,y+h-1,clr,1.0
''End Sub
''private Sub UI_DrawRect(ByVal x As Integer, ByVal y As Integer, Byval w As Integer, ByVal h As Integer, ByVal clr As Integer)
''	YaglGfxDevice_box x,y,x+w-1,y+h-1,clr,1.0
''End Sub
private Sub UI_DrawFillRect alias "UI_DrawFillRect" (ByVal x As Integer, ByVal y As Integer, Byval w As Integer, ByVal h As Integer, ByVal clr As Integer)
	YaglGfxDevice_solidbox x,y,x+w-1,y+h-1,clr,1.0
End Sub

Sub UI_DrawCheckBox alias "UI_DrawCheckBox" (ByVal r As UI_RECT_T Ptr, ByVal Checked As Integer, ByVal clr As Integer)
	Dim r1 As UI_RECT_T
	UI_SetRect @r1, r->x, r->y + (r->h - r->w) \ 2, r->w, r->w
	If (Checked) Then
		UI_DrawFrame @r1, UI_FRAMESTYLE_THIN or UI_FRAMESTYLE_FILL or UI_FS_BTN_DN
	else
		UI_DrawFrame @r1, UI_FRAMESTYLE_THIN or UI_FRAMESTYLE_FILL or UI_FS_BTN_UP
	End If
End Sub

Sub UI_DrawFrame alias "UI_DrawFrame"  (ByVal r As UI_RECT_T Ptr, ByVal iStyle as integer)
	Dim As Integer c1=0,c2=0
	Dim As Integer x = r->x
	Dim As Integer y = r->y
	Dim As Integer w = r->w
	Dim As Integer h = r->h

	select case (iStyle And 7)
	case UI_FRAMESTYLE_UP
		DrawBorder x,y,w,h,brdrup,th,1.0,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case UI_FRAMESTYLE_DOWN
		DrawBorder x,y,w,h,brdrdn,th,1.0,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case UI_FRAMESTYLE_THIN, (UI_FRAMESTYLE_THIN or UI_FRAMESTYLE_UP)
		DrawBorder x,y,w,h,squareup,th,1.0,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case (UI_FRAMESTYLE_THIN or UI_FRAMESTYLE_DOWN)
		DrawBorder x,y,w,h,squaredn,th,1.0,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case UI_FRAMESTYLE_EDGE
		DrawBorder x,y,w,h,brdrdn,th,1.0,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	case UI_FRAMESTYLE_THICK
		DrawBorder x,y,w,h,brdrup,th,1.0,(iStyle And UI_FRAMESTYLE_FILL), (iStyle And UI_FRAMESTYLE_OUTSIDE),UI_GetSysColor(UI_SYSCLR_FRAME_FACE)

	end Select

End Sub

Sub UI_DrawCaret alias "UI_DrawCaret" (byval x as integer, byval y as integer, byval w as integer, byval h as integer, byval clr as integer)
	UI_DrawFillRect x, y, w, h, clr
End Sub
